// -----------------------------------------------------------------------------
//
// Author(s):     Peter Schraut (www.console-dev.de)
// Last Modified: 6th July 2005
//
// Purpose:       Demo shows how to use the map-system with dynamic tile
//                reloading enabled as well as animated tiles.
//
//                Please see MapDynamicTileReloading before.
//
// -----------------------------------------------------------------------------
#include <hel.h>


// -----------------------------------------------------------------------------
// Defines
// -----------------------------------------------------------------------------
#define NUMBER_TILES_ROM    (53)
#define NUMBER_TILES_RAM    (53)

// Size of map in tiles
#define WORLD_WIDTH         (1024/8)
#define WORLD_HEIGHT        (256/8)

// -----------------------------------------------------------------------------
// Global Variables
// -----------------------------------------------------------------------------
u8 g_NewFrame=TRUE;
u32 g_TotalFrames=0;
u32 g_HeartFrames=0;
TMapScrollInfo g_MapInfo ATTR_MEM_IN_EWRAM;
u16 g_BufferRom[NUMBER_TILES_ROM] ATTR_MEM_IN_EWRAM;
u16 g_BufferRam[NUMBER_TILES_RAM*3] ATTR_MEM_IN_EWRAM;


// -----------------------------------------------------------------------------
// External Data
// -----------------------------------------------------------------------------
extern const unsigned short bg_Palette[256];
extern const unsigned short world3_layer0_Map[WORLD_WIDTH*WORLD_HEIGHT];
extern const unsigned char world3_layer0_Tiles[3392];
extern const unsigned char heart_Tiles[512];

// -----------------------------------------------------------------------------
// Prototypes
// -----------------------------------------------------------------------------
void VBLInterruptHandler(void);
void AnimateHeartGraphic(void);

// -----------------------------------------------------------------------------
// Program Entry Point
// -----------------------------------------------------------------------------
int main(void)
{
    ham_Init();
    hel_PadInit();
    ham_SetBgMode(0);
    
    // Init Tile-System for Background 0
    hel_TileInit(0,                   // BgNo
                 world3_layer0_Tiles, // Source tiledata
                 NUMBER_TILES_ROM,    // Number of tiles in ROM
                 g_BufferRom,         // BufferA
                 NUMBER_TILES_RAM,    // Maximum number of tiles in RAM
                 g_BufferRam,         // BufferB
                 1,                   // Tileset colormode (0=16colors, 1=256colors)
                 0,                   // Palettenumber in 16color mode (0..15)
                 TRUE);               // CBB only mode
                                
    // Init Map-System for Background 0
    g_MapInfo = hel_MapInit(0,                  // BgNo
                            WORLD_WIDTH,        // MapWidth in tiles
                            WORLD_HEIGHT,       // MapHeight in tiles
                            FALSE,              // RotationMap?
                            (void*)world3_layer0_Map);// Source MapData
 
    // Turn on Dynamic-Tile-Reloading
    hel_MapSetDynamicTileReloading(&g_MapInfo, TRUE);
    
    // Load Background Palette
    hel_PalBgLoad256((void*)bg_Palette);
    
    // Turn on Background
    ham_InitBg(0, TRUE, 0, FALSE);

    // Start VBL Interrupt
    ham_StartIntHandler(INT_TYPE_VBL, (void*)&VBLInterruptHandler);
                       
   
    
    // Infinite loop
    for(;;)
    {
        // New Frame?
        if(g_NewFrame)
        {
            // Read current pad state
            hel_PadReadState();

            #define SCROLL_SPEED 2
            TPoint32 ScrollBy={0, 0};
            
            // Query Pad to see in what direction
            // the map should be scrolled
            if(hel_Pad.Held.Down)
            {
               ScrollBy.Y += SCROLL_SPEED;
            }
            else
            if(hel_Pad.Held.Up)
            {
               ScrollBy.Y -= SCROLL_SPEED;
            }

            if(hel_Pad.Held.Left)
            {
               ScrollBy.X -= SCROLL_SPEED;
            }
            else
            if(hel_Pad.Held.Right)
            {
               ScrollBy.X += SCROLL_SPEED;
            }
            
            // Animate tiles
            AnimateHeartGraphic();
            
            // Now Scroll the map!
            hel_MapScrollBy(&g_MapInfo, ScrollBy.X, ScrollBy.Y);
            
            // Frame done!
            g_NewFrame=FALSE;
        }
    }
    
    return 0;
}

// -----------------------------------------------------------------------------
// Animate heart graphic
//
// Check if the tile is currently loaded. If it is not, it is not visible
// and can be skipped. The worldmap has in the top left corner the heart, which
// we use as reference tilenumber, because the tilenumber is for every heart the same.
// -----------------------------------------------------------------------------
void AnimateHeartGraphic(void)
{
  // Get number of tile specified by X and Y from our worldmap.
  #define GetWorldMapTile(X,Y)    world3_layer0_Map[((Y)*WORLD_WIDTH)+(X)]
  
  // Get pointer to one of the heart tiles.
  // Check "demos\SharedMedia\gfx\heart.bmp" to see how the graphic is
  // organised. The graphic has 2 tiles on X and 4 tiles on Y.
  // We do Y*128, because 1 tile in 256 colors is 64bytes and
  // because we want to skip 2 tiles (remember, 2 tiles on X) we
  // multiply by 128!
  #define GetHeartTile(X,Y)       &heart_Tiles[((X)*64)+((Y)*128)]
  
  // The rownumber where each heart starts
  u32 RowNo = (g_HeartFrames&1)*2;
  
  
   // Is the top-left tile of the heart loaded?
  if(hel_TileIsGraphicLoaded(0, GetWorldMapTile(0,0)))
  {
    // YES, it is! So now reload the graphic!
    hel_TileReloadGraphic(0, GetWorldMapTile(0,0), GetHeartTile(0, RowNo));
  }

  // Is the top-right tile of the heart loaded?
  if(hel_TileIsGraphicLoaded(0, GetWorldMapTile(1,0)))
  {
    hel_TileReloadGraphic(0, GetWorldMapTile(1,0), GetHeartTile(1, RowNo));
  }

   // Is the bottom-left tile of the heart loaded?
  if(hel_TileIsGraphicLoaded(0, GetWorldMapTile(0,1)))
  {
    hel_TileReloadGraphic(0, GetWorldMapTile(0,1), GetHeartTile(0, RowNo+1));
  }

   // Is the bottom-right tile of the heart loaded?
  if(hel_TileIsGraphicLoaded(0, GetWorldMapTile(1,1)))
  {
    hel_TileReloadGraphic(0, GetWorldMapTile(1,1), GetHeartTile(1, RowNo+1));
  }
}

// -----------------------------------------------------------------------------
// VBL Interrupt Handler
// -----------------------------------------------------------------------------
void VBLInterruptHandler()
{
	g_NewFrame=TRUE;
	g_TotalFrames++;

	// Only increase g_HeartFrames every 16th frame
	if(!(g_TotalFrames&15))
    g_HeartFrames++;
}

