// -----------------------------------------------------------------------------
//
// Author(s):     Peter Schraut (www.console-dev.de)
// Last Modified: 5th July 2005
//
// Purpose:       Demo shows how to use the map-system with dynamic tile
//                reloading enabled.
//
//                HEL's tilesystem supports 4 and 8 bit graphics, but supports
//                currently only non rotation map. It expects 16bit tilenumbers,
//                which means you cannot use flipped tiles.
//                If you use gfx2gba to convert graphics, make sure to specify
//                -F to avoid flipped tiles. Here is how the image is converted:
//                gfx2gba -t8 -m -F -fsrc -ogfx -pvampire.pal ../SharedMedia/gfx/vampire.bmp
//
//                gfx2gba v0.14 has no option to prevent it when creating 4bit
//                graphics to store the palettenumber in each tile. Therefore
//                will tile corruption occur when you use gfx2gba with 4bit graphics
//                combined with HEL. I already asked gfx2gba author to implement
//                an option for this. Hope he implements it soon.
//
//                Output from gfx2gba:
//
//                =======================
//                gfx2gba Converter v0.14
//                -----------------------
//                (C) 2oo1-2oo2 [TRiNiTY]
//                =======================
//
//                Reading: ../SharedMedia/gfx/vampire.bmp (800x600 pixel, 256 colors)
//                Number of tiles before optimization: 7500
//                Number of tiles after  optimization: 7438
//
//                ***WARNING: number of tiles (7438) exceeds 1024!
//                Saving tiled bitmap data to: gfx\vampire.raw.c ... ok
//                Saving map data to: gfx\vampire.map.c ... ok
//
//                Saving masterpalette to..: gfx\vampire.pal.c ... ok
//
//                Total files read & converted.:   1
//                Colors used before converting: 255
//                Colors used after converting.: 256
//                Colors saved.................:  -1
//
//
//
//                "Number of tiles" becomes important later in this example.
//
// -----------------------------------------------------------------------------
#include <hel.h>


// -----------------------------------------------------------------------------
// Defines
// -----------------------------------------------------------------------------
// This equals "Number of tiles after  optimization"
#define NUMBER_TILES_ROM    (7438)    

// Because the image has so many unique tiles, there
// can be in worst case 20*30 tiles on the screen at
// the same time. We advance both values by 1 to have
// some buffer to precache tiles.
#define NUMBER_TILES_RAM    (21*31)

// Size of map in tiles
#define VAMPIRE_WIDTH       (800/8)
#define VAMPIRE_HEIGHT      (600/8)

// -----------------------------------------------------------------------------
// Global Variables
// -----------------------------------------------------------------------------
u8 g_NewFrame=TRUE;
TMapScrollInfo g_MapInfo ATTR_MEM_IN_EWRAM;

// This buffer is used to resolve Rom to Ram tilenumbers.
u16 g_BufferRom[NUMBER_TILES_ROM] ATTR_MEM_IN_EWRAM;

// This buffer is to resolve Ram to Rom tilenumbers,
// reference counting and a "stack" to pop and push
// tileslots.
u16 g_BufferRam[NUMBER_TILES_RAM*3] ATTR_MEM_IN_EWRAM;


// -----------------------------------------------------------------------------
// External Data
// -----------------------------------------------------------------------------
extern const unsigned short vampire_Map[7500];
extern const unsigned short vampire_Palette[256];
extern const unsigned char vampire_Tiles[476032];


// -----------------------------------------------------------------------------
// Prototypes
// -----------------------------------------------------------------------------
void VBLInterruptHandler(void);


// -----------------------------------------------------------------------------
// Program Entry Point
// -----------------------------------------------------------------------------
int main(void)
{
    ham_Init();
    hel_PadInit();
    ham_SetBgMode(0);
    
    // Init Tile-System for Background 0
    hel_TileInit(0,                // BgNo
                 vampire_Tiles,    // Source tiledata
                 NUMBER_TILES_ROM, // Number of tiles in ROM
                 g_BufferRom,      // BufferA
                 NUMBER_TILES_RAM, // Maximum number of tiles in RAM
                 g_BufferRam,      // BufferB
                 1,                // Tileset colormode (0=16colors, 1=256colors)
                 0,                // Palettenumber in 16color mode (0..15)
                 TRUE);            // CBB only mode
                                
    // Init Map-System for Background 0
    g_MapInfo = hel_MapInit(0,                  // BgNo
                            VAMPIRE_WIDTH,      // MapWidth in tiles
                            VAMPIRE_HEIGHT,     // MapHeight in tiles
                            FALSE,              // RotationMap?
                            (void*)vampire_Map);// Source MapData
 
    // Turn on Dynamic-Tile-Reloading
    hel_MapSetDynamicTileReloading(&g_MapInfo, TRUE);
    
    // Load Background Palette
    hel_PalBgLoad256((void*)vampire_Palette);
    
    // Turn on Background
    ham_InitBg(0, TRUE, 0, FALSE);

    // Start VBL Interrupt
    ham_StartIntHandler(INT_TYPE_VBL, (void*)&VBLInterruptHandler);
                       
   
    
    // Infinite loop
    for(;;)
    {
        // New Frame?
        if(g_NewFrame)
        {
            // Read current pad state
            hel_PadReadState();

            #define SCROLL_SPEED 2
            TPoint32 ScrollBy={0, 0};
            
            // Query Pad to see in what direction
            // the map should be scrolled
            if(hel_Pad.Held.Down)
            {
               ScrollBy.Y += SCROLL_SPEED;
            }
            else
            if(hel_Pad.Held.Up)
            {
               ScrollBy.Y -= SCROLL_SPEED;
            }

            if(hel_Pad.Held.Left)
            {
               ScrollBy.X -= SCROLL_SPEED;
            }
            else
            if(hel_Pad.Held.Right)
            {
               ScrollBy.X += SCROLL_SPEED;
            }
            
            // Now Scroll the map!
            hel_MapScrollBy(&g_MapInfo, ScrollBy.X, ScrollBy.Y);
            
            // Frame done!
            g_NewFrame=FALSE;
        }
    }
    
    return 0;
}


// -----------------------------------------------------------------------------
// VBL Interrupt Handler
// -----------------------------------------------------------------------------
void VBLInterruptHandler()
{
	g_NewFrame=TRUE;
}

